#extension GL_OES_EGL_image_external : require

precision mediump float;

//arithmetic supports n views however texture currently only supports 4 views in a quad configuration.
const float numViews = 4.0;

// A stereo camera texture, in horizontal side-by-side layout.
uniform sampler2D cameraTex;

// The disparity map. Note that this is monocular (not side-by-side).
uniform sampler2D disparityTex;

// Texture coordinates, ranging from (0, 0) at the bottom-left of the screen,
//   to (1, 1) at the top-right of the screen.
varying vec2 vTextureCoord;

// Gain = 3D effect multiplier.
uniform float uGain;

vec4 sampleFromLeft(vec2 coord) {
    if (coord.x < 0.0 || coord.x > 1.0 || coord.y < 0.0 || coord.y > 1.0) {
        return vec4(0.0, 0.0, 0.0, 0.0);
    }
    return texture2D(cameraTex, vec2(coord.x / 2.0, coord.y));
}

vec4 sampleFromRight(vec2 coord) {
    if (coord.x < 0.0 || coord.x > 1.0 || coord.y < 0.0 || coord.y > 1.0) {
        return vec4(0.0, 0.0, 0.0, 0.0);
    }
    return texture2D(cameraTex, vec2(coord.x / 2.0 + 0.5, coord.y));
}

/**
 * This shader program takes in a side-by-side stereo texture and a disparity map,
 *  and outputs a 4-view texture in 2x2 format.
 */
void main() {
    // Sample from our disparity map, but align to the output 4x4 layout.
    vec2 disparityCoord = mod(vTextureCoord * 2.0, 1.0);

    float disparity = texture2D(disparityTex, disparityCoord).r;

    // Map from [0,1] to percentage disparity [-.1, .1]
    disparity = (disparity - .5) * .2;

    // splits quad in to 0-1 for X and 1/0 for y;
    vec2 halfId = floor(vTextureCoord * 2.0);

    // calculate view id 0,1,2,3 for top left, right, bottom left, right
    float viewId = halfId.x + 2.0 * halfId.y;

    // calculate transformation to center left/right disparity
    float inputToCenter = vTextureCoord.y > 0.5 ? 0.5 : -0.5;

    // calculate multiplication based on view from center (-1.5, -0.5, 0.5, 1.5)
    float centerToOutput = viewId - (numViews - 1.0)/2.0;

    // calculate effective disparity based on E(disparity) = (disp_to_center * disparity  + center_to_disp * f(disparity)
    // where f() = gain*disparity
    float effectiveDisparity = inputToCenter * disparity - (centerToOutput * uGain * disparity);

    //choose sample from left/right image based on location in quad
    vec2 viewCoord = disparityCoord * .5;

    // when backward mapping we do x - effective disparity
    vec2 sampleCoord = vec2(viewCoord.x * 2.0 - effectiveDisparity, viewCoord.y * 2.0);

    // view 0,1 sample from left and view 2,3 sample from right
    gl_FragColor = vTextureCoord.y < 0.5 ? sampleFromLeft(sampleCoord) : sampleFromRight(sampleCoord);
}
